/**
 * Java
 *
 * Copyright 2012 IS2T. All rights reserved.
 * For demonstration purpose only
 * IS2T PROPRIETARY. Use is subject to license terms.
 */
package com.is2t.mwt.tutorial.ex16;

import ej.microui.Listener;
import ej.microui.Model;

/**
 * A simulation of a mass on a spring. The equilibrium position is 0.
 */
public class SpringSimulation extends Model implements Listener {
	
	// defaults chosen to give a period of approx 200
	private static final double DEFAULT_SPRING_CONSTANT = 0.098696;
	private static final int DEFAULT_MASS_KG = 100;

	private int position;
	private IntHolderAndListener mass = new IntHolderAndListener(DEFAULT_MASS_KG);
	private double springConstant = DEFAULT_SPRING_CONSTANT;
	private double amplitude = 0;
	private int time = 0;
	private double w;

	/**
	 * Create a new spring-mass system that is at rest (position = 0)
	 */
	public SpringSimulation () {
		position = 0;
		w = calculateW();
		mass.addListener(new Listener() {
			public void performAction(int value) {
				setMass(value);
			}
			public void performAction(int value, Object object) {}
			public void performAction() {}
		});
	}

	/**
	 * Set the position of the mass. This position becomes the amplitude of the 
	 * oscillation. Resets the simulation time to 0.
	 * @param pos the position
	 */
	public void setPosition(int pos) {
		amplitude = pos;
		position = pos;
		time = 0;
		changed(getPosition());
	}

	/**
	 * Advance the simulation by one time step
	 */
	public void step() {
		time++;
		position = calculatePosition();
		changed(getPosition());
	}

	public int getPosition() {
		return position;
	}
	
	/**
	 * Set the simulation time. This will cause the position to change.
	 * @param time
	 */
	public void setTime(int time) {
		this.time = time;
		position = calculatePosition();
	}

	public int getPeriod() {
		return (int) ((2 * Math.PI * Math.sqrt(mass.get() / springConstant))+0.5); //round the result
	}

	public IntHolderAndListener getMass() {
		return mass;
	}

	/**
	 * Set the position of the mass to the specified value.
	 */
	public void performAction(int value) {
		setPosition(value);
	}

	/**
	 * Unused - does nothing
	 */
	public void performAction() {}

	/**
	 * Unused - does nothing
	 */
	public void performAction(int value, Object object) {}

	private void setMass(int value) {
		mass.set(value);
		w = calculateW();
		position = calculatePosition();
	}

	private double calculateW() {
		return Math.sqrt(springConstant / mass.get());
	}

	private int calculatePosition() {
		double position = amplitude * Math.cos(w * time);
		return ((int) (position+(position>=0?0.5:-0.5))); // round the result
	}

}
